/*
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
*/

#include "gamestatsimpl.h"

#include "dealer.h"
#include "version.h"

#include <QDebug>

#include <QtGlobal>

#include <QSettings>
#include <QPushButton>
#include <QList>


GameStatsImpl::GameStatsImpl(QWidget* aParent)
	: QDialog(aParent),
	  indexToIdMap()
{
	ui = new Ui::GameStats();
	ui->setupUi(this);

	setWindowTitle(tr("Statistics"));
	
	ui->GameType->setFocus();
	ui->GameType->setMaxVisibleItems(DealerInfoList::self()->games().size());

	QMap<QString,int> nameToIdMap;
	foreach (DealerInfo* game, DealerInfoList::self()->games())
		nameToIdMap.insert(QObject::tr(game->name), game->ids.first());
	
	QMap<QString,int>::const_iterator it = nameToIdMap.constBegin();
	QMap<QString,int>::const_iterator end = nameToIdMap.constEnd();
	for (; it != end; ++it)
	{
		// Map combobox indices to game indices
		indexToIdMap[ui->GameType->count()] = it.value();
		ui->GameType->addItem(it.key());
	}

	showGameType(indexToIdMap[0]);

	connect(ui->GameType, SIGNAL(activated(int)), SLOT(selectionChanged(int)));
	QPushButton* resetButton = ui->buttonBox->button(QDialogButtonBox::Reset);
	connect(resetButton, SIGNAL(clicked()), SLOT(resetStats()));
	connect(ui->buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
}

void GameStatsImpl::selectionChanged(int comboIndex)
{
	int gameIndex = indexToIdMap[comboIndex];
	setGameType(gameIndex);
}

void GameStatsImpl::showGameType(int gameIndex)
{
	int comboIndex = indexToIdMap.key(gameIndex);
	ui->GameType->setCurrentIndex(comboIndex);
	setGameType(gameIndex);
}

void GameStatsImpl::setGameType(int gameIndex)
{
	QSettings cg; //(KGlobal::config(), scores_group);
	unsigned int t = cg.value(QString("total%1").arg(gameIndex),0).toUInt();
	ui->Played->setText(QString::number(t));
	unsigned int w = cg.value(QString("won%1").arg(gameIndex),0).toUInt();
	if (t)
		ui->Won->setText(QString("%1 (%2%)").arg(w).arg(w*100/t));
	else
		ui->Won->setText( QString::number(w));
	ui->WinStreak->setText( QString::number( cg.value(QString("maxwinstreak%1").arg(gameIndex), 0).toInt()));
	ui->LoseStreak->setText( QString::number( cg.value(QString("maxloosestreak%1").arg(gameIndex), 0).toInt()));
	unsigned int l = cg.value(QString("loosestreak%1").arg(gameIndex),0).toUInt();
	if (l)
		ui->CurrentStreak->setText( tr("%n loss(es)", "", l) );
	else
		ui->CurrentStreak->setText( tr("%n win(s)", "", cg.value(QString("winstreak%1").arg(gameIndex), 0).toUInt()) );
}

void GameStatsImpl::resetStats()
{
	int gameIndex = indexToIdMap[ui->GameType->currentIndex()];
	Q_ASSERT(gameIndex >= 0);
	QSettings cg; //(KGlobal::config(), scores_group);
	cg.setValue(QString("total%1").arg(gameIndex),0);
	cg.setValue(QString("won%1").arg(gameIndex),0);
	cg.setValue(QString("maxwinstreak%1").arg(gameIndex),0);
	cg.setValue(QString("maxloosestreak%1").arg(gameIndex),0);
	cg.setValue(QString("loosestreak%1").arg(gameIndex),0);
	cg.setValue(QString("winstreak%1").arg(gameIndex),0);
	cg.sync();

	setGameType(gameIndex);
}
