/*
     patience -- main program
      Copyright (C) 1995  Paul Olav Tvete <paul@troll.no>

 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.

 */

#include "cardmaps.h"
#include "dealer.h"
#include "version.h"
#include "pwidget.h"
#include "patsolve/patsolve.h"

#include <QDebug>

#include <QApplication>
#include <QTranslator>
#include <QLocale>
#include <QLibraryInfo>
#include <QMenuBar>
#include <QUrl>

#include <QtCore/QFile>
#include <QtCore/QTime>
#include <QtCore/QTimer>
#include <QtGui/QResizeEvent>
#include <QtXml/QDomDocument>

#include <cstdio>
#include <climits>

//static const char description[] = QT_TR_NOOP("KDE Patience Game");

/*static DealerScene *getDealer( int wanted_game )
{
    DealerInfo *di = 0;
    for (QList<DealerInfo*>::ConstIterator it = DealerInfoList::self()->games().begin();
         it != DealerInfoList::self()->games().end(); ++it)
    {
        if ( (*it)->hasId(wanted_game) ) {
            di = *it;
            DealerScene *f = di->createGame();
            if ( !f || !f->solver() ) {
                qCritical() << "There is no solver for" << di->name;
                return 0;
            }

            fprintf( stdout, "Testing %s\n", di->name );
            return f;
        }
    }
    return 0;
}

// A function to remove all nonalphanumeric characters from a string
// and convert all letters to lowercase.
QString lowerAlphaNum( const QString & string )
{
    QString result;
    for ( int i = 0; i < string.size(); ++i )
    {
        QChar c = string.at( i );
        if ( c.isLetterOrNumber() )
            result += c.toLower();
    }
    return result;
}*/

// A function that forces all pending resize events to happen immediately.
// Used to prevent expensive resize events after the scene has been created.
void sendAllPendingResizeEvents(QWidget * widget)
{
	bool foundOne;
	do
	{
		foundOne = false;
		QList<QWidget *> allChildWidgets = widget->findChildren<QWidget *>();
		allChildWidgets.prepend(widget);
		foreach(QWidget* w, allChildWidgets)
		{
			if (w->testAttribute(Qt::WA_PendingResizeEvent))
			{
				QResizeEvent e(w->size(), QSize());
				QApplication::sendEvent(w, &e);
				w->setAttribute(Qt::WA_PendingResizeEvent, false);
				// hack: make QTabWidget think it's visible; no layout otherwise
				w->setAttribute(Qt::WA_WState_Visible, true);
				foundOne = true;
			}
		}
		// Process LayoutRequest events, in particular
		qApp->sendPostedEvents();

		if (!foundOne)
		{
			// Reset visible flag, to avoid crashes in qt
			foreach(QWidget* w, allChildWidgets)
			w->setAttribute(Qt::WA_WState_Visible, false);
		}
	} while (foundOne);
}

int main(int argc, char **argv)
{
	/*    KAboutData aboutData( "kpat", 0, tr("KPatience"),
	                          KPAT_VERSION, tr(description), KAboutData::License_GPL,
	                          tr("(c) 1995, Paul Olav Tvete\n"
	                          "(c) 2000 Stephan Kulow"), KLocalizedString(), "http://games.kde.org/kpat" );
	    aboutData.addAuthor(tr("Paul Olav Tvete"),KLocalizedString(),"paul@troll.no");
	    aboutData.addAuthor(tr("Mario Weilguni"),KLocalizedString(),"mweilguni@kde.org");
	    aboutData.addAuthor(tr("Matthias Ettrich"),KLocalizedString(),"ettrich@kde.org");
	    aboutData.addAuthor(tr("Rodolfo Borges"),tr("Some Game Types"),"barrett@9hells.org");
	    aboutData.addAuthor(tr("Peter H. Ruegg"),KLocalizedString(),"kpat@incense.org");
	    aboutData.addAuthor(tr("Michael Koch"), tr("Bug fixes"), "koch@kde.org");
	    aboutData.addAuthor(tr("Marcus Meissner"), tr("Shuffle algorithm for game numbers"),
	                        "mm@caldera.de");
	    aboutData.addAuthor(tr("Dr. Tom"), tr("Patience Solver"), "http://members.tripod.com/professor_tom/");
	    aboutData.addAuthor(tr("Stephan Kulow"), tr("Rewrite and current maintainer"),
	                        "coolo@kde.org");
	    aboutData.addAuthor(tr("Erik Sigra"), tr("Improved Klondike"), "sigra@home.se");
	    aboutData.addAuthor(tr("Josh Metzler"), tr("Spider Implementation"), "joshdeb@metzlers.org");
	    aboutData.addAuthor(tr("Maren Pakura"), tr("Documentation"), "maren@kde.org");
	    aboutData.addAuthor(tr("Inge Wallin"), tr("Bug fixes"), "inge@lysator.liu.se");
	    aboutData.addAuthor(tr("Simon Hürlimann"), tr("Menu and Toolbar work"), "simon.huerlimann@huerlisi.ch");

	    // Create a KComponentData earlier than normal so that we can use i18n to translate
	    // the names of the game types in the help text.
	    KComponentData componentData(&aboutData);
	    KGlobal::locale()->insertCatalog("libkdegames");

	    QMap<QString, int> indexMap;
	    QStringList gameList;
	    foreach ( const DealerInfo *di, DealerInfoList::self()->games() )
	    {
	        const QString translatedKey = lowerAlphaNum( QObject::tr( di->name ) );
	        gameList << translatedKey;
	        indexMap.insert( translatedKey, di->ids.first() );
	        indexMap.insert( lowerAlphaNum( QString( di->name ) ), di->ids.first() );
	    }
	    gameList.sort();

	    qDebug() << "game list  = " << gameList;

	    KCmdLineArgs::init( argc, argv, &aboutData );

	    KCmdLineOptions options;
	    options.add("solvegame <file>", tr( "Try to find a solution to the given savegame" ) );
	    options.add("solve <num>", tr("Dealer to solve (debug)" ));
	    options.add("start <num>", tr("Game range start (default 0:INT_MAX)" ));
	    options.add("end <num>", tr("Game range end (default start:start if start given)" ));
	    options.add("gametype <game>", tr("Skip the selection screen and load a particular game type. Valid values are: %1").subs(gameList.join(i18nc("List separator", ", "))));
	    options.add("+file", tr("File to load"));
	    KCmdLineArgs::addCmdLineOptions (options);
	    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();*/

	QApplication app(argc, argv);

	app.setOrganizationName("rusbitech");
	app.setApplicationName("qpat");

	QString localeName = QLocale::system().name();

	QTranslator qtTranslator;
	qtTranslator.load("qt_" + localeName, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
	app.installTranslator(&qtTranslator);

	QTranslator translator;
	translator.load(QString(DATADIR) + "/qpat_" + localeName);
	app.installTranslator(&translator);

	app.setWindowIcon(QIcon::fromTheme("qpat"));

	/* QString savegame = args->getOption( "solvegame" );
	if ( !savegame.isEmpty() )
	{
	    QFile of(savegame);
	    of.open(QIODevice::ReadOnly);
	    QDomDocument doc;
	    doc.setContent(&of);

	    cardMap c;
	    DealerScene *f = getDealer( doc.documentElement().attribute("id").toInt() );

	    f->openGame( doc );
	    f->solver()->translate_layout();
	    int ret = f->solver()->patsolve();
	    if ( ret == Solver::WIN )
	        fprintf( stdout, "won\n");
	    else if ( ret == Solver::NOSOL )
	        fprintf( stdout, "lost\n" );
	    else
	        fprintf( stdout, "unknown\n");

	    return 0;
	}

	bool ok = false;
	int wanted_game = -1;
	if ( args->isSet( "solve" ) )
	    wanted_game = args->getOption("solve").toInt( &ok );
	if ( ok )
	{
	    ok = false;
	    int end_index = -1;
	    if ( args->isSet( "end" ) )
	        end_index = args->getOption("end").toInt( &ok );
	    if ( !ok )
	        end_index = -1;
	    ok = false;
	    int start_index = -1;
	    if ( args->isSet( "start" ) )
	        start_index = args->getOption("start").toInt( &ok );
	    if ( !ok ) {
	        start_index = 0;
	        end_index = INT_MAX;
	    } else {
	        if ( end_index == -1 )
	            end_index = start_index;
	    }
	    cardMap c;
	    DealerScene *f = getDealer( wanted_game );
	    if ( !f )
	        return 1;

	    QTime mytime;
	    for ( int i = start_index; i <= end_index; i++ )
	    {
	        mytime.start();
	        f->setGameNumber( i );
	        f->restart();
	        f->solver()->translate_layout();
	        int ret = f->solver()->patsolve();
	        if ( ret == Solver::WIN )
	            fprintf( stdout, "%d won (%d ms)\n", i, mytime.elapsed() );
	        else if ( ret == Solver::NOSOL )
	            fprintf( stdout, "%d lost (%d ms)\n", i, mytime.elapsed()  );
	        else
	            fprintf( stdout, "%d unknown (%d ms)\n", i, mytime.elapsed() );
	    }
	    fprintf( stdout, "all_moves %ld\n", all_moves );
	    return 0;
	}

	QString gametype = args->getOption("gametype").toLower();
	QFile savedState( KStandardDirs::locateLocal("appdata", saved_state_file) );*/

	pWidget *w = new pWidget;
	sendAllPendingResizeEvents(w->menuBar());
	/*    if (args->count())
	    {
	        if (!w->openGame(args->url(0)))
	            w->slotShowGameSelectionScreen();
	    }
	    else if (indexMap.contains(gametype))
	    {
	        w->slotGameSelected(indexMap.value(gametype));
	    }
	     else if (savedState.exists())
	    {
	        if (!w->openGame(savedState.fileName(), false))
	            w->slotShowGameSelectionScreen();
	        savedState.remove();
	    }
	    else*/
	{
		w->slotShowGameSelectionScreen();
	}
	w->show();

//    args->clear();
	return app.exec();
}
