/* -*- C++ -*-
 *
 * patience -- main program
 *   Copyright (C) 1995  Paul Olav Tvete
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
 *
 *
 * Heavily modified by Mario Weilguni <mweilguni@sime.com>
 *
 */

#ifndef PWIDGET_H
#define PWIDGET_H

#include <QUrl>
#include <QMainWindow>
#include <QMap>

class QAction;
class QLabel;
class QStackedWidget;

class cardMap;
class DealerInfo;
class DealerScene;
class DemoBubbles;
class PatienceView;

class pWidget: public QMainWindow {
    Q_OBJECT

public:
    pWidget();
    ~pWidget();
    
    void addGameOptionsAction(QAction* a);

public slots:
    void undoMove();
    void redoMove();
    void newGameType(int id);
    void startRandom();
    void restart();
    void slotShowGameSelectionScreen();

    void openGame();
    bool openGame(const QString& filePath, bool addToRecentFiles = true);
    void saveGame();

    void newGame();
    void chooseGame();
    void helpGame();
    void enableAutoDrop(bool enable);
    void enableSolver(bool enable);
    void enableRememberState(bool enable);
    void toggleDemoAction(bool active);
    void showStats();

    void slotGameSolverStart();
    void slotGameSolverWon();
    void slotGameSolverLost();
    void slotGameSolverUnknown();
    void slotGameLost();
    void slotUpdateMoves(int moves);

    void slotPickRandom();
    void slotSelectDeck();

    void slotSnapshot();

    void slotGameSelected(int id);

protected:
    virtual void closeEvent(QCloseEvent * e);
    virtual void saveNewToolbarConfig();

private slots:
    void slotSnapshot2();

private:
    void setGameCaption();
    void startNew(long gameNumber);
    void updateActions();
    void updateGameActionList();
    
    void setupGUI();
    
    void removeGameActions();

    QList<QAction*> gameActions;
    QAction* quitSeparator;

    QAction* newGameAction;
    QAction* restartGameAction;
    QAction* loadGameAction;
    QAction* saveGameAction;
    QAction* quitGameAction;
    
    QAction* chooseGameAction;
    QAction* changeGameTypeAction;
    QAction* randomSetAction;
    QAction* selectDeckAction;
    QAction* gameStatsAction;

    // Members
    QAction        *undo;
    QAction        *redo;
    QAction        *demoaction;
    QAction        *hintaction;
    QAction        *drawaction;
    QAction        *dealaction;
    QAction        *redealaction;
    QAction        *dropaction;
    QAction        *gamehelpaction;
    QAction        *autodropaction;
    QAction        *solveraction;
    QAction        *rememberstateaction;
    //TODO KRecentFilesAction  *recent;

    cardMap        *m_cards; // possibly move to PatienceView
    QMap<int, const DealerInfo*>  m_dealer_map;
    QMap<int, const DealerInfo*>::const_iterator  m_dealer_it;
    int m_freeCellId;

    QStackedWidget *m_stack;
    PatienceView   *dill;
    DealerScene    *m_dealer;
    DemoBubbles    *m_bubbles;

    QLabel         *solverStatus;
    QLabel         *moveStatus;

    QMenu* gameMenu;
    QMenu* settingsMenu;
    QToolBar* gameToolBar;
};

#endif
